/*------------------------------------------------------------------------------
 bin/djv_view/image_cache.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include "image_cache.h"
#include <djv_image/image.h>
#include <algorithm>

namespace djv_view {
namespace image_cache {

Ref::Ref(djv_image::Image * in, const void * key, int frame) :
  _image(in),
  _key(key),
  _frame(frame),
  _ref_count(1)
{}

djv_image::Image * Ref::get() { return _image.get(); }

void Ref::key(const void * in)
{
  _key = in;
}

const void * Ref::key() const { return _key; }

int Ref::frame() const { return _frame; }

void Ref::ref()
{
  ++_ref_count;
}
  
void Ref::ref_del()
{
  --_ref_count;
}

int Ref::ref_count() const { return _ref_count; }

Cache::Cache() :
  _cache_max(256 * mem::megabyte),
  _cache_size(0)
{
  DJV_DBG_OBJ("Cache::Cache");
}

Cache::~Cache()
{
  DJV_DBG_OBJ("Cache::~Cache");
#if defined(DJV_DBG)
  dbg();
#endif
  
  for (uint i = 0; i < _ref_list.size(); ++i)
    delete _ref_list[i];
}

Ref * Cache::create(djv_image::Image * image, const void * key, int frame)
{
  DJV_DBG_OBJ("Cache::create");
  DJV_DBG_PRN("image = " << *image);
  DJV_DBG_PRN("key = " << int(key));
  DJV_DBG_PRN("frame = " << frame);
#if defined(DJV_DBG)
  dbg();
#endif

  Ref * out = new Ref(image, key, frame);
  _ref_list += out;
  _cache_size += out->get()->bytes_data();

  if (_cache_size > _cache_max) purge();

#if defined(DJV_DBG)
  dbg();
#endif
  return out;
}

Ref * Cache::get(const void * key, int frame)
{
  DJV_DBG_OBJ("Cache::get");
  DJV_DBG_PRN("key = " << int(key));
  DJV_DBG_PRN("frame = " << frame);
#if defined(DJV_DBG)
  //dbg();
#endif

  Ref * out = 0;
  for (List<Ref *>::iterator i = _ref_list.begin(); i != _ref_list.end(); ++i)
    if (key == (*i)->key() && frame == (*i)->frame())
    {
      out = *i;
      out->ref();
      _ref_list.erase(i);
      _ref_list += out;
      break;
    }

#if defined(DJV_DBG)
  dbg();
#endif

  DJV_DBG_PRN("out = " << (out ? int(out->key()) : 0));
  return out;
}

void Cache::max(int in)
{
  DJV_DBG_OBJ("Cache::max");
  DJV_DBG_PRN("in = " << in);
#if defined(DJV_DBG)
  dbg();
#endif
  
  _cache_max = in * mem::megabyte;
  
  //if (_cache_size > _cache_max)
    purge();
  
#if defined(DJV_DBG)
  dbg();
#endif
}

int Cache::max() const
{
  return _cache_max / mem::megabyte;
}

int Cache::size() const
{
  return _cache_size / mem::megabyte;
}

void Cache::del(const void * key)
{
  DJV_DBG_OBJ("Cache::del");
  DJV_DBG_PRN("key = " << int(key));
#if defined(DJV_DBG)
  dbg();
#endif

  for (uint i = 0; i < _ref_list.size(); ++i)
    if (key == _ref_list[i]->key())
      _ref_list[i]->key(0);

  purge();
  
#if defined(DJV_DBG)
  dbg();
#endif
}

void Cache::del()
{
  DJV_DBG_OBJ("Cache::del");
#if defined(DJV_DBG)
  dbg();
#endif

  List<Ref *> tmp(_ref_list.size());
  
  int size = 0;
  for (uint i = 0; i < _ref_list.size(); ++i)
    if (! _ref_list[i]->ref_count())
    {
      _cache_size -= _ref_list[i]->get()->bytes_data();
      delete _ref_list[i];
    }
    else
    {
      tmp[size++] = _ref_list[i];
    }

  tmp.resize(size);
  _ref_list = tmp;

#if defined(DJV_DBG)
  dbg();
#endif
}
  
List<int> Cache::frame_list(const void * key)
{
  List<int> out;
  for (uint i = 0; i < _ref_list.size(); ++i)
    if (key == _ref_list[i]->key()) out += _ref_list[i]->frame();
  std::sort(out.begin(), out.end());
  return out;
}

void Cache::purge()
{
  DJV_DBG_OBJ("Cache::purge");
#if defined(DJV_DBG)
  //dbg();
#endif

  List<Ref *> tmp(_ref_list.size());
  
  int size = 0;
  for (uint i = 0; i < _ref_list.size(); ++i)
    if (_cache_size > _cache_max && ! _ref_list[i]->ref_count())
    {
      _cache_size -= _ref_list[i]->get()->bytes_data();
      delete _ref_list[i];
    }
    else
    {
      tmp[size++] = _ref_list[i];
    }

  tmp.resize(size);
  _ref_list = tmp;
  
#if defined(DJV_DBG)
  //dbg();
#endif
}

void Cache::dbg()
{
  DJV_DBG_OBJ("Cache::dbg");
  DJV_DBG_PRN("refs = " << _ref_list.size());
  DJV_DBG_PRN("cache max = " << uint(_cache_max / mem::megabyte));
  DJV_DBG_PRN("cache size = " << uint(_cache_size / mem::megabyte));
  
  for (uint i = 0; i < _ref_list.size(); ++i)
    DJV_DBG_PRN(
      "item (ref = " << _ref_list[i]->ref_count() << ") = " <<
      int(_ref_list[i]->key()) << " " <<
      _ref_list[i]->frame());
}

const List<int> & default_size()
{
  static const List<int> data = List<int>() <<
    256 <<
    512 <<
    1024 <<
    2048 <<
    4096;
  return data;
}

const List<Str> & label_size()
{
  static const List<Str> data = List<Str>() <<
    "256" <<
    "512" <<
    "1024" <<
    "2048" <<
    "4096";
  return data;
}

}
}

