/*------------------------------------------------------------------------------
 include/djv_base/file.h
 
 File System Information
 -----------------------
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_FILE_H
#define DJV_FILE_H

#include <djv_base/base.h>

namespace djv_base {
namespace file {

//------------------------------------------------------------------------------
// File Sequence
  
struct Seq
{
  Seq();
  Seq(const List<int> &, int pad = 0);
  Seq(int start, int end, int pad = 0);
  
  List<int> list;
  int pad;

  void set(int start, int end);
  inline int start() const;
  inline int end() const;
};

//------------------------------------------------------------------------------
// File Information

enum TYPE
{
  FILE,
  SEQ,
  DIR
};

enum PERM
{
  READ  = 1,
  WRITE = 2,
  EXEC  = 4
};

class File
{
public:

  File();
  File(const Str &, bool stat = true);

  // Value.
  
  void set(const Str &, bool stat = true);
  Str get(int frame = -1, bool path = true) const;
  Str name() const;
  
  // Components.
  
  Str path, base, number, extension;

  // Information.

  bool stat(const Str & path = Str());
  
  TYPE type;
  uint64_t size;
  uid_t user;
  int perm;
  time_t time;
  
  // Sequence.

  inline bool seq_add(const File &);
  inline bool seq_valid() const;
  inline bool seq_wildcard() const;
  
  Seq seq;

  // Conversion.

  inline operator Str() const;

private:

  void init();
};

//------------------------------------------------------------------------------
// Functions

// Split a file name into pieces.

void split(const Str &, Str * path, Str * base, Str * number, Str * extension);

// Check if file exists.

bool exists(const File &);

// Sequence.

void seq_sort(Seq *);

enum SEQ_COMPRESS
{
  SEQ_OFF,
  SEQ_SPARSE,
  SEQ_RANGE
};

void seq_compress(List<File> *, SEQ_COMPRESS = SEQ_RANGE);

List<Str> seq_expand(const File &);

int seq_find(int, const List<int> &);

File seq_match(const File &, const List<File> &);

// Filter.

enum FILTER
{
  FILTER_NONE = 0,
  FILTER_FILE = 1,
  FILTER_DIR = 2,
  FILTER_HIDDEN = 4
};

void filter(
  List<File> *,
  int filter,
  const List<Str> & glob_list = List<Str>()
);

// Sort.

enum SORT
{
  SORT_NAME,
  SORT_TYPE,
  SORT_SIZE,
  SORT_USER,
  SORT_PERM,
  SORT_TIME
};

void sort(
  List<File> *,
  SORT,
  bool reverse = false
);

List<File> sort_dir(const List<File> &);

// Paths.

Str  path_absolute(const Str &);
bool path_is_absolute(const Str &);
Str  path_fix(const Str &);
//Str path_posix(const Str &);

static List<char> path_separator_list = List<char>() << '/' << '\\';
static List<char> list_separator_list = List<char>() << ':' << ';';
#if defined(WIN32)
static char & path_separator = path_separator_list[1];
static char & list_separator = list_separator_list[1];
#else
static char & path_separator = path_separator_list[0];
static char & list_separator = list_separator_list[0];
#endif

//------------------------------------------------------------------------------
// Comparison

inline bool operator == (const Seq &, const Seq &);
inline bool operator != (const Seq &, const Seq &);

inline bool operator == (const File &, const File &);
inline bool operator != (const File &, const File &);

//------------------------------------------------------------------------------
// Conversion

Str seq_to_str(const Seq &);
Seq str_to_seq(const Str &);

Str frame_to_str(int, int pad = 0);
inline int str_to_frame(const Str &, int * pad = 0);

Str & operator >> (Str &, Seq &) throw (Str);
Str & operator >> (Str &, SORT &) throw (Str);
Str & operator >> (Str &, SEQ_COMPRESS &) throw (Str);
Str & operator << (Str &, const Seq &);
Str & operator << (Str &, SORT);
Str & operator << (Str &, SEQ_COMPRESS);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_type();
Str label_size(uint64_t);
const Str & label_perm(int);
const List<Str> & label_sort();
const List<Str> & label_seq_compress();

//------------------------------------------------------------------------------
// Debugging

Dbg & operator << (Dbg &, const Seq &);
Dbg & operator << (Dbg &, const File &);

}

//------------------------------------------------------------------------------
// Declarations

using file::File;

}

#include <djv_base/file_.h>

#endif

