/*------------------------------------------------------------------------------
 include/djv_base/str.h
 
 Strings
 -------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_STR_H
#define DJV_STR_H

#include <djv_base/list.h>
#include <string>

namespace djv_base {
namespace str {

//------------------------------------------------------------------------------
// Constants

static const int cstr_size = 256;

//------------------------------------------------------------------------------
// String

class Str : public std::string
{
public:

  inline Str();
  inline Str(const Str &, int position = 0, int size = npos);
  inline Str(const std::string &);
  inline Str(const char *);
  inline Str(const char *, int size);
  inline Str(char, int size = 1);
};

//------------------------------------------------------------------------------
// Formatting

// Example:
//
// Str str = str::Format("str = %%, int = %%, double = %%").
//   arg("abc").
//   arg(123).
//   arg(456.0);

class Format
{
public:

  Format(const Str &);

  // Arguments.
  
  Format arg(const Str &, int width = 0, char fill = ' ') const;
  Format arg(const List<Str> &, const Str & separator = " ", int width = 0,
    char fill = ' ') const;
  Format arg(int, int width = 0, char fill = ' ') const;
  Format arg(uint, int width = 0, char fill = ' ') const;
  Format arg(double, int width = 0, int precision = 0, char fill = ' ') const;
  
  // Operators.
  
  operator Str () const;
  
private:

  Str _value;
};

//------------------------------------------------------------------------------
// Functions

// Case.

inline Str lower(const Str &);
inline Str upper(const Str &);

// Join.

inline Str join(const List<Str> &, const Str & separator = Str());
inline Str join(const List<Str> &, char separator);

// Split.

List<Str> split(const Str &, const Str & separator, bool keep_empty = false);
List<Str> split(const Str &, char separator, bool keep_empty = false);
List<Str> split(const Str &, const List<Str> & separator,
  bool keep_empty = false);
List<Str> split(const Str &, const List<char> & separator,
  bool keep_empty = false);

// Replace.

inline Str replace(const Str &, char match, char replace);

// Formatting.

//#if defined(WIN32)
//#define SNPRINTF ::sprintf_s
//#else
#define SNPRINTF ::snprintf
//#endif

List<Str> format(const Str &, int width);

//------------------------------------------------------------------------------
// Comparison

inline bool is_digit(char);

// Case-insensitive compare.

inline bool compare(const Str &, const Str &);

//------------------------------------------------------------------------------
// Conversion

int cstr(const Str &, char *, int max_len = 0, bool terminate = true);

template<typename T>
inline T str_to_int(const char *, int max_len = cstr_size);
inline int str_to_int(const Str &);
inline uint str_to_uint(const Str &);
double str_to_float(const Str &);

template<typename T>
inline int int_to_str(T, char *, int max_len = cstr_size);
inline Str int_to_str(int);
inline Str uint_to_str(uint);
Str float_to_str(double, int precision = 0);

bool serialize(Str *, Str *);
bool serialize(Str *, uint *, const List<Str> & label);
bool serialize(Str *, Str *, const List<Str> & label);
template <typename T>
inline bool serialize(Str *, T *, const List<Str> & label);
Str serialize(const Str &);
template <typename T>
inline Str serialize(T, const List<Str> & label);

Str & operator >> (Str &, Str &) throw (Str);
Str & operator >> (Str &, bool &) throw (Str);
Str & operator >> (Str &, int &) throw (Str);
Str & operator >> (Str &, uint &) throw (Str);
Str & operator >> (Str &, double &) throw (Str);
template <typename T>
inline Str & operator >> (Str &, List<T> &) throw (Str);

Str & operator << (Str &, const Str &);
Str & operator << (Str &, bool);
Str & operator << (Str &, int);
Str & operator << (Str &, uint);
Str & operator << (Str &, double);
template <typename T>
inline Str & operator << (Str &, const List<T> &);

#define DJV_STR_OPERATOR_SET(TYPE, LABEL) \
  \
  Str & operator >> (Str & in, TYPE & out) throw (Str) \
  { \
    if (! str::serialize(&in, &out, LABEL)) throw in; \
    return in; \
  }

#define DJV_STR_OPERATOR_GET(TYPE, LABEL) \
  \
  Str & operator << (Str & out, TYPE in) \
  { \
    return out << str::serialize(in, LABEL); \
  }

//------------------------------------------------------------------------------
// Labels

template<typename T>
inline Str label(const T &);

const List<Str> & label_bool();

}
}

#include <djv_base/str_.h>

#endif

