/*------------------------------------------------------------------------------
 include/djv_base/vec.h
 
 Vectors
 -------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VEC_H
#define DJV_VEC_H

#include <djv_base/base.h>

namespace djv_base {
namespace vec {

//------------------------------------------------------------------------------
// Vector Base

template<typename T, int D>
struct Vec
{
  inline Vec();
  inline Vec(const Vec<T, D> &);
  inline Vec(T);
  Vec<T, D> & operator = (const Vec<T, D> &);
  
  // Value.
  
  static const int dimension;
  T e [D];
  
  inline void set(const Vec<T, D> &);
  inline void set(T);
  
  // Functions.
  
  inline void zero();
  
  inline Vec<T, D> & operator += (const Vec<T, D> &);
  inline Vec<T, D> & operator -= (const Vec<T, D> &);
  inline Vec<T, D> & operator *= (const Vec<T, D> &);
  inline Vec<T, D> & operator /= (const Vec<T, D> &);

  inline Vec<T, D> & operator += (T);
  inline Vec<T, D> & operator -= (T);
  inline Vec<T, D> & operator *= (T);
  inline Vec<T, D> & operator /= (T);
};

//------------------------------------------------------------------------------
// Two-Dimensional Vector

template<typename T>
struct Vec<T, 2>
{
  inline Vec();
  inline Vec(const Vec<T, 2> &);
  inline Vec(T, T);
  inline Vec(T);
  inline Vec<T, 2> & operator = (const Vec<T, 2> &);
  
  // Value.
  
  static const int dimension;
  T e [2];
  T & x, & y;

  inline void set(const Vec<T, 2> &);
  inline void set(T);
  
  // Functions.
  
  inline void zero();
  
  inline Vec<T, 2> & operator += (const Vec<T, 2> &);
  inline Vec<T, 2> & operator -= (const Vec<T, 2> &);
  inline Vec<T, 2> & operator *= (const Vec<T, 2> &);
  inline Vec<T, 2> & operator /= (const Vec<T, 2> &);
  
  inline Vec<T, 2> & operator += (T);
  inline Vec<T, 2> & operator -= (T);
  inline Vec<T, 2> & operator *= (T);
  inline Vec<T, 2> & operator /= (T);
  
  inline operator Vec<int, 2> () const;
  inline operator Vec<double, 2> () const;
};

typedef Vec<bool, 2> Vec2b;
typedef Vec<int, 2> Vec2i;
typedef Vec<double, 2> Vec2f;

//------------------------------------------------------------------------------
// Three-Dimensional Vector

template<typename T>
struct Vec<T, 3>
{
  inline Vec();
  inline Vec(const Vec<T, 3> &);
  inline Vec(T, T, T);
  inline Vec(T);
  inline Vec<T, 3> & operator = (const Vec<T, 3> &);
  
  // Value.
  
  static const int dimension;
  T e [3];
  T & x, & y, & z;
  
  inline void set(const Vec<T, 3> &);
  inline void set(T);
  
  // Functions.
  
  inline void zero();
  
  inline Vec<T, 3> & operator += (const Vec<T, 3> &);
  inline Vec<T, 3> & operator -= (const Vec<T, 3> &);
  inline Vec<T, 3> & operator *= (const Vec<T, 3> &);
  inline Vec<T, 3> & operator /= (const Vec<T, 3> &);
  
  inline Vec<T, 3> & operator += (T);
  inline Vec<T, 3> & operator -= (T);
  inline Vec<T, 3> & operator *= (T);
  inline Vec<T, 3> & operator /= (T);

  inline operator Vec<int, 3> () const;
  inline operator Vec<double, 3> () const;
};

typedef Vec<bool, 3> Vec3b;
typedef Vec<int, 3> Vec3i;
typedef Vec<double, 3> Vec3f;

//------------------------------------------------------------------------------
// Functions

// Absolute value.

template<typename T, int D>
inline Vec<T, D> abs(const Vec<T, D> &);

// Swap components.

template<typename T, int D>
inline Vec<T, D> swap(const Vec<T, D> &);

// Minimum and maximum.

template<typename T, int D>
inline Vec<T, D> min(const Vec<T, D> &, const Vec<T, D> &);
template<typename T, int D>
inline Vec<T, D> max(const Vec<T, D> &, const Vec<T, D> &);
template<typename T, int D>
inline Vec<T, D> clamp(const Vec<T, D> &, const Vec<T, D> &, const Vec<T, D> &);

// Greater than zero.

template<typename T, int D>
inline bool size_valid(const Vec<T, D> &);

// Length.

template<typename T, int D>
inline double length(const Vec<T, D> &);

// Normalize.

template<typename T, int D>
inline Vec<T, D> normalize(const Vec<T, D> &);

// Dot product.

template<typename T, int D>
inline double dot(const Vec<T, D> &, const Vec<T, D> &);

// Aspect ratio.

template<typename T, int D>
inline double aspect(const Vec<T, D> &);

// Arithmetic.

template<typename T, int D>
inline Vec<T, D> operator + (const Vec<T, D> &, const Vec<T, D> &);
template<typename T, int D>
inline Vec<T, D> operator - (const Vec<T, D> &, const Vec<T, D> &);
template<typename T, int D>
inline Vec<T, D> operator * (const Vec<T, D> &, const Vec<T, D> &);
template<typename T, int D>
inline Vec<T, D> operator / (const Vec<T, D> &, const Vec<T, D> &);

template<typename T, int D>
inline Vec<T, D> operator + (const Vec<T, D> &, T);
template<typename T, int D>
inline Vec<T, D> operator - (const Vec<T, D> &, T);
template<typename T, int D>
inline Vec<T, D> operator * (const Vec<T, D> &, T);
template<typename T, int D>
inline Vec<T, D> operator / (const Vec<T, D> &, T);

// Negate.

template<typename T, int D>
inline Vec<T, D> operator - (const Vec<T, D> &);

//------------------------------------------------------------------------------
// Comparison

template<typename T>
inline bool operator == (const T &, const T &);
template<typename T>
inline bool operator != (const T &, const T &);

template<typename T>
inline bool operator <  (const T &, const T &);
template<typename T>
inline bool operator <= (const T &, const T &);

template<typename T>
inline bool operator >  (const T &, const T &);
template<typename T>
inline bool operator >= (const T &, const T &);

//------------------------------------------------------------------------------
// Conversion

template<typename T, typename T2, int D>
inline Vec<T2, D> convert(const Vec<T, D> &);

template<typename T, typename T2, int D>
inline Vec<T2, D> ceil(const Vec<T, D> &);
template<typename T, typename T2, int D>
inline Vec<T2, D> floor(const Vec<T, D> &);

void spherical_to_cartesian(const Vec3f &, Vec3f *);
void cartesian_to_spherical(const Vec3f &, Vec3f *);

template<typename T, int D>
inline Str & operator >> (Str &, Vec<T, D> &) throw (Str);
template<typename T, int D>
inline Str & operator << (Str &, const Vec<T, D> &);

//------------------------------------------------------------------------------
// Debugging

template<typename T>
inline Dbg & operator << (Dbg &, const T &);

}

//------------------------------------------------------------------------------
// Declarations

using vec::Vec;
using vec::Vec2b;
using vec::Vec2i;
using vec::Vec2f;
using vec::Vec3b;
using vec::Vec3i;
using vec::Vec3f;

}

#include <djv_base/vec_.h>
#include <djv_base/vec2_.h>
#include <djv_base/vec3_.h>
#include <djv_base/vec_etc_.h>

#endif

