/*------------------------------------------------------------------------------
 include/djv_base/vec_etc_.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_base/math.h>

namespace djv_base {
namespace vec {

template<typename T, int D>
inline Vec<T, D> abs(const Vec<T, D> & in)
{
  Vec<T, D> out;
  for (int i = 0; i < in.dimension; ++i)
    out.e[i] = math::abs(in.e[i]);
  return out;
}

template<typename T, int D>
inline Vec<T, D> swap(const Vec<T, D> & in)
{
  Vec<T, D> out;
  for (int i = 0; i < in.dimension; ++i)
    out.e[i] = in.e[in.dimension - 1 - i];
  return out;
}

template<typename T, int D>
inline Vec<T, D> min(const Vec<T, D> & a, const Vec<T, D> & b)
{
  Vec<T, D> out;
  for (int i = 0; i < a.dimension; ++i)
    out.e[i] = math::min(a.e[i], b.e[i]);
  return out;
}

template<typename T, int D>
inline Vec<T, D> max(const Vec<T, D> & a, const Vec<T, D> & b)
{
  Vec<T, D> out;
  for (int i = 0; i < a.dimension; ++i)
    out.e[i] = math::max(a.e[i], b.e[i]);
  return out;
}

template<typename T, int D>
inline Vec<T, D> clamp(
  const Vec<T, D> & in,
  const Vec<T, D> & min,
  const Vec<T, D> & max
) {
  Vec<T, D> out;
  for (int i = 0; i < in.dimension; ++i)
    out.e[i] = math::clamp(in.e[i], min.e[i], max.e[i]);
  return out;
}

template<typename T, int D>
inline bool size_valid(const Vec<T, D> & in)
{
  return in > Vec<T, D>(0);
}

template<typename T, int D>
inline double length(const Vec<T, D> & in)
{
  double out = 0.0;
  for (int i = 0; i < in.dimension; ++i)
    out += in.e[i] * in.e[i];
  return math::sqrt(out);
}

template<typename T, int D>
inline Vec<T, D> normalize(const Vec<T, D> & in)
{
  Vec<T, D> out;
  const double l = length(in);
  if (l != 0.0)
    for (int i = 0; i < in.dimension; ++i)
      out.e[i] = T(in.e[i] / l);
  return out;
}

template<typename T, int D>
inline double dot(const Vec<T, D> & a, const Vec<T, D> & b)
{
  double out = 0.0;
  for (int i = 0; i < a.dimension; ++i)
    out += a.e[i] * b.e[i];
  return out;
}

template<typename T, int D>
inline double aspect(const Vec<T, D> & in)
{
  return in.e[1] != T(0) ? (in.e[0] / double(in.e[1])) : 0.0;
}

#define _VEC_FNC_OP(IN) \
  \
  template<typename T, int D> \
  inline Vec<T, D> operator IN (const Vec<T, D> & a, const Vec<T, D> & b) \
  { \
    Vec<T, D> out; \
    for (int i = 0; i < a.dimension; ++i) \
      out.e[i] = a.e[i] IN b.e[i]; \
    return out; \
  }

_VEC_FNC_OP(+)
_VEC_FNC_OP(-)
_VEC_FNC_OP(*)
_VEC_FNC_OP(/)

#define _VEC_FNC_OP2(IN) \
  \
  template<typename T, int D> \
  inline Vec<T, D> operator IN (const Vec<T, D> & a, T b) \
  { \
    Vec<T, D> out; \
    for (int i = 0; i < a.dimension; ++i) \
      out.e[i] = a.e[i] IN b; \
    return out; \
  }

_VEC_FNC_OP2(+)
_VEC_FNC_OP2(-)
_VEC_FNC_OP2(*)
_VEC_FNC_OP2(/)

template<typename T, int D>
inline Vec<T, D> operator - (const Vec<T, D> & in)
{
  Vec<T, D> out;
  for (int i = 0; i < in.dimension; ++i)
    out.e[i] = -in.e[i];
  return out;
}

#define _VEC_COMPARE(IN) \
  \
  template<typename T> \
  inline bool operator IN (const T & a, const T & b) \
  { \
    for (int i = 0; i < a.dimension; ++i) \
      if (! (a.e[i] IN b.e[i])) return false; \
    return true; \
  }

_VEC_COMPARE(==)
_VEC_COMPARE(<)
_VEC_COMPARE(<=)
_VEC_COMPARE(>)
_VEC_COMPARE(>=)

template<typename T>
inline bool operator != (const T & a, const T & b) { return ! (a == b); }

template<typename T, typename T2, int D>
inline Vec<T2, D> convert(const Vec<T, D> & in)
{
  Vec<T2, D> out;
  for (int i = 0; i < in.dimension; ++i)
    out.e[i] = T2(in.e[i]);
  return out;
}

template<typename T, typename T2, int D>
inline Vec<T2, D> ceil(const Vec<T, D> & in)
{
  Vec<T2, D> out;
  for (int i = 0; i < in.dimension; ++i)
    out.e[i] = math::ceil(in.e[i]);
  return out;
}

template<typename T, typename T2, int D>
inline Vec<T2, D> floor(const Vec<T, D> & in)
{
  Vec<T2, D> out;
  for (int i = 0; i < in.dimension; ++i)
    out.e[i] = math::floor(in.e[i]);
  return out;
}

template<typename T, int D>
inline Str & operator << (Str & out, const Vec<T, D> & in)
{
  for (int i = 0; i < in.dimension; ++i)
    out << in.e[i];
  return out;
}

template<typename T, int D>
inline Str & operator >> (Str & in, Vec<T, D> & out) throw (Str)
{
  for (int i = 0; i < out.dimension; ++i)
    in >> out.e[i];
  return in;
}

template<typename T>
inline Dbg & operator << (Dbg & dbg, const T & in)
{
  return dbg << str::label(in);
}

}
}

