/*------------------------------------------------------------------------------
 include/djv_image/io_ppm.h
 
 Image I/O - PPM
 ---------------
 
 NetPBM image file format.
 
 Support:
 
 * Image data: L, RGB, 1-bit, 8-bit, 16-bit.

 References:
 
 * Netpbm, "PPM Format Specification",
   http://netpbm.sourceforge.net/doc/ppm.html

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_IO_PPM_H
#define DJV_IMAGE_IO_PPM_H

#include <djv_image/io.h>

namespace djv_image {
namespace io_ppm {

//------------------------------------------------------------------------------
// Information

static const Str name = "PPM";

static const List<Str> extension_list = List<Str>() <<
  ".ppm" <<
  ".pnm" <<
  ".pgm" <<
  ".pbm";

//------------------------------------------------------------------------------
// Options

enum TYPE
{
  TYPE_AUTO,
  U1
};

enum MODE
{
  ASCII,
  BINARY
};

enum SAVE_OPTION
{
  SAVE_TYPE,
  SAVE_MODE
};

struct Save_Option
{
  Save_Option() :
    type(TYPE_AUTO),
    mode(BINARY)
  {}

  TYPE type;
  MODE mode;
};

//------------------------------------------------------------------------------
// Load Plugin

class Load : public io::Load
{
public:

  Load();
  
  Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  void open(const File &, Info *, Tag *, Time *) throw (Error);
  void open(const File &, List<Info> *, Tag *, Time *) throw (Error);
  const Image * get(int frame, int layer, data::PROXY) throw (Error);

private:

  void _open(const File &, Info *) throw (Error);

  File _file;
  djv_base::Io _io;
  int _bit_depth;
  MODE _mode;
  Image _image;
  Data _tmp;
};

//------------------------------------------------------------------------------
// Save Plugin

class Save : public io::Save
{
public:

  Save();
  
  Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  bool option(const Str &, Str *);
  Str option(const Str &) const;
  const List<Str> & option_list() const;

  void open(const File &, const Info &, const Tag &, const Time &)
    throw (Error);
  void set(const Image &, int frame) throw (Error);

private:
  
  void _open(const File &) throw (Error);

  Save_Option _option;
  File _file;
  djv_base::Io _io;
  int _bit_depth;
  Info _info;
  Image _image;
};

//------------------------------------------------------------------------------
// Functions

size_t bytes_scanline(int width, int channels, int bit_depth, MODE);

void ascii_load(
  djv_base::Io *,
  void *,
  int size,
  int bit_depth
) throw (Error);

size_t ascii_save(
  const void *,
  void *,
  int size,
  int bit_depth
);

//------------------------------------------------------------------------------
// Conversion

Str & operator >> (Str &, TYPE &) throw (Str);
Str & operator >> (Str &, MODE &) throw (Str);
Str & operator << (Str &, TYPE);
Str & operator << (Str &, MODE);

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_type();
const List<Str> & label_mode();
const List<Str> & label_save_option();

}
}

#endif

