/*------------------------------------------------------------------------------
 include/djv_ui/dialog.h
 
 Dialogs
 -------
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_DIALOG_H
#define DJV_UI_DIALOG_H

#include <djv_ui/window.h>

namespace djv_ui {
namespace dialog {

//------------------------------------------------------------------------------
// Dialog Base

class Base : public window::Widget
{
public:
  
  Base(const Str & title = Str());
  virtual ~Base() = 0;
};

//------------------------------------------------------------------------------
// Message Dialog Base

class Message_Base : public Base
{
public:

  Message_Base(const Str & title = Str());
  virtual ~Message_Base() = 0;
  
  // Layout.
  
  virtual void dirty();
  virtual const Vec2i & size_min() const;
  
private:

  Vec2i _size_min;
};

//------------------------------------------------------------------------------
// Message Dialog

class Message : public Message_Base
{
public:
  
  Message(const Str & label = Str());
  virtual ~Message();
  
  // Label.
 
  void label(const Str &);
  const Str & label() const;
  
  // Widget.
  
  virtual void show();

private:

  DJV_CALLBACK(Message, close_callback, bool);

  struct _Message;
  std::auto_ptr<_Message> _p;
};

//------------------------------------------------------------------------------
// Question Dialog

class Question : public Message_Base
{
public:
  
  Question(const Str & label = Str());
  virtual ~Question();

  // Value.
  
  Signal<bool> sig;
  
  // Label.

  void label(const Str &);
  const Str & label() const;
  
  // Widget.
  
  virtual void show();

private:

  DJV_CALLBACK(Question, ok_callback, bool);
  DJV_CALLBACK(Question, close_callback, bool);

  struct _Question;
  std::auto_ptr<_Question> _p;
};

//------------------------------------------------------------------------------
// Input Dialog

class Input : public Message_Base
{
public:
  
  Input(const Str & label = Str(), const Str & value = Str());
  virtual ~Input();
  
  // Value.

  void set(const Str &);
  Str get() const;
  Signal<const Str &> sig;
  
  // Label.
  
  void label(const Str &);
  const Str & label() const;
  
  // Widget.
  
  virtual void show();
  
private:

  DJV_CALLBACK(Input, widget_callback, const Str &);
  DJV_CALLBACK(Input, ok_callback, bool);
  DJV_CALLBACK(Input, close_callback, bool);

  struct _Input;
  std::auto_ptr<_Input> _p;
};

//------------------------------------------------------------------------------
// Choice Dialog

class Choice : public Message_Base
{
public:
  
  Choice(const Str & label = Str(), const List<Str> & list = List<Str>());
  virtual ~Choice();
  
  // Value.

  Signal<int> sig;
  
  // List.

  void list(const List<Str> &);
  List<Str> list() const;
  
  // Label.
  
  void label(const Str &);
  const Str & label() const;
  
  // Widget.
  
  virtual void show();

private:

  DJV_CALLBACK(Choice, ok_callback, bool);
  DJV_CALLBACK(Choice, close_callback, bool);

  struct _Choice;
  std::auto_ptr<_Choice> _p;
};

//------------------------------------------------------------------------------
// Progress Dialog

class Progress : public Message_Base
{
public:
  
  Progress();
  virtual ~Progress();
  virtual void del();
  
  // Value.

  void set(int);
  Signal<int> sig;
  Signal<bool> del_sig;

  // Label.
  
  void label(const Str &);
  const Str & label() const;

private:

  DJV_CALLBACK(Progress, cancel_callback, bool);
  DJV_FL_CALLBACK(Progress, idle_callback);

  int _value, _current;
  djv_base::time::Timer _time;
  double _time_accum;
  djv_base::time::Timer _elapsed;
  
  struct _Progress;
  std::auto_ptr<_Progress> _p;
};

//------------------------------------------------------------------------------
// Color Dialog

class Color : public Base
{
public:
  
  Color(const djv_image::Color & = djv_image::Color());
  virtual ~Color();
  
  // Value.

  void set(const djv_image::Color &);
  const djv_image::Color & get() const;
  Signal<const djv_image::Color &> sig;

private:

  DJV_CALLBACK(Color, widget_callback, const djv_image::Color &);
  DJV_CALLBACK(Color, close_callback, bool);
  
  void widget_update();
  
  djv_image::Color _value;
  
  struct _Color;
  std::auto_ptr<_Color> _p;
};

//------------------------------------------------------------------------------
// Labels

extern const Str
  label_message,
  label_question,
  label_input,
  label_choice,
  label_progress,
  label_progress_estimate,
  label_progress_elapsed,
  label_color,
  label_ok,
  label_close,
  label_cancel,
  label_yes,
  label_no,
  label_show,
  label_pin;

//------------------------------------------------------------------------------
// Global

class Global
{
public:

  Global();
  ~Global();

  void message(const Str &) const;

  void question(
    const Str & label,
    Callback_Base *,
    Signal<bool>::Callback *
  ) const;

  void input(
    const Str & label,
    const Str & value,
    Callback_Base *,
    Signal<const Str &>::Callback *
  ) const;

  void choice(
    const Str & label,
    const List<Str> & list,
    Callback_Base *,
    Signal<int>::Callback *
  ) const;

  void progress(
    const Str & label,
    int value,
    Callback_Base *,
    Signal<int>::Callback * callback,
    Signal<bool>::Callback * del_callback
  ) const;
  void progress_del(Callback_Base *) const;

  void color(
    const djv_image::Color &,
    Callback_Base *,
    Signal<const djv_image::Color &>::Callback *
  ) const;

private:

  widget::Auto_Ptr<Message> _message;
  widget::Auto_Ptr<Question> _question;
  widget::Auto_Ptr<Input> _input;
  widget::Auto_Ptr<Choice> _choice;
  widget::Auto_Ptr<Progress> _progress;
  widget::Auto_Ptr<Color> _color;
};

Global * global();

}
}

#endif

