/*------------------------------------------------------------------------------
 lib/djv_base/str_etc.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_base/dbg.h>
#include <djv_base/math.h>
#include <djv_base/mem.h>
#include <djv_base/str.h>
#include <stdlib.h>

namespace djv_base {
namespace str {

List<Str> split(const Str & in, const Str & separator, bool keep_empty)
{
  return split(in, List<Str>() << separator, keep_empty);
}

List<Str> split(const Str & in, const List<Str> & separator, bool keep_empty)
{
  List<Str> out;

  const uint size = in.size();
  uint i = 0, j = 0;
  for (; i < size; ++i)
    for (uint k = 0; k < separator.size(); ++k)
    {
      const uint separator_size = separator[k].size();
      if (0 == in.compare(i, separator_size, separator[k]))
      {
        if (i != j)
          out += Str(in, j, i - j);
        else if (keep_empty)
          out += Str();
        i += separator_size - 1;
        j = i + 1;
        break;
      }
    }

  if (j < size) out += Str(in, j, i - j);
  else if (! size) out += Str();

  return out;
}

List<Str> split(const Str & in, char separator, bool keep_empty)
{
  return split(in, List<char>() << separator, keep_empty);
}

List<Str> split(const Str & in, const List<char> & separator, bool keep_empty)
{
  List<Str> out;
    
  const uint size = in.size();
  uint i = 0, j = 0;
  for (; i < size; ++i)
    for (uint k = 0; k < separator.size(); ++k)
      if (in[i] == separator[k])
      {
        if (i != j) out += Str(in, j, i - j);
        else if (keep_empty) out += Str();
        j = i + 1;
        break;
      }

  if (j < size) out += Str(in, j, i - j);
  else if (! size) out += Str();

  return out;
}

List<Str> format(const Str & in, int width)
{
  width = math::max(width, 1);
  
  List<Str> out;
  
  // Split into lines.
  
  const List<Str> line_list = str::split(in, '\n', true);
  for (uint i = 0; i < line_list.size(); ++i)
  {
    // Split into words.
    
    const List<Str> word_list = str::split(line_list[i], ' ', true);
    for (uint j = 0; j < word_list.size(); ++j)
    {
      // Make new lines according to width.
      
      Str line;
      for (uint k = 0; j < word_list.size(); ++k, ++j)
      {
        if (k) line += ' ';
        line += word_list[j];
        if (j < word_list.size() - 1)
          if (int(line.size() + 1 + word_list[j + 1].size()) >= width) break;
      }
      out += line;
    }
  }

  return out;
}

int cstr(const Str & in, char * out, int max_len, bool terminate)
{
  const int size = max_len ?
    math::min(int(in.size()), max_len - int(terminate)) :
    int(in.size() + int(terminate));

  int i = 0;
  for (; i < size; ++i)
    out[i] = in[i];
  if (terminate)
    out[i++] = 0;
  return i;
}

double str_to_float(const Str & in)
{
  return atof(in.c_str());
}

Str float_to_str(double in, int precision)
{  
  char tmp [cstr_size] = "";
  int size = 0;
  
  if (precision)
  {
    char format [] = "%0. f";
    format[3] = '0' + abs(precision);
    size = SNPRINTF(tmp, cstr_size, format, in);
  }
  else
  {
    size = SNPRINTF(tmp, cstr_size, "%0.3f", in);
  }

  if (precision <= 0 && size)
  {
    int i = size - 1;
    for (; i >= 0 && '0' == tmp[i]; --i, --size) ;
    if (i)
      if ('.' == tmp[i]) --size;
  }
  return Str(tmp, (size > 0) ? size : 0);
}

namespace {

const Str serialize_null = "null";
const char serialize_separator = '|';

}

bool serialize(Str * in, Str * out)
{
  const uint size = in->size();
  if (! size) return false;
  
  // Find word.
  
  uint start = 0;
  for (; ' ' == (*in)[start] && start < size; ++start) ;
  uint end = start;
  for (; ' ' != (*in)[end] && end < size; ++end) ;
  
  // Extract word.
  
  const Str tmp = Str(*in, start, end - start);
  if (tmp != serialize_null)
    //*out = tmp;
    *out = replace(tmp, serialize_separator, ' ');
  *in = Str(*in, end, size - end + 1);
  return true;
}

bool serialize(Str * in, uint * out, const List<Str> & label)
{
  const uint size = in->size();
  if (! size) return false;
  
  // XXX Case-insensitive?

  //const Str _in = lower(*in);

  // XXX Extra space prevents partial matches.
  
  const Str _in = lower(*in) + " ";
  
  // Find word.
  
  uint start = 0;
  for (; ' ' == _in[start] && start < size; ++start) ;

  // Extract word.
  
  const uint label_size = label.size();
  for (uint i = 0; i < label_size; ++i)
  {
    //if (0 == _in.compare(start, label[i].size(), lower(label[i])))
    const Str tmp = replace(Str(_in, start, label[i].size() + 1),
      serialize_separator, ' ');
    if (compare(tmp, label[i] + " "))
    {
      *out = i;
      const uint end = start + label[i].size();
      *in = Str(*in, end, size - end + 1);
      return true;
    }
  }
  
  return false;
}

bool serialize(Str * in, Str * out, const List<Str> & label)
{
  uint i = 0;
  if (! serialize(in, &i, label)) return false;
  *out = label[i];
  return true;
}

Str serialize(const Str & in)
{
  //return in.size() ? in : serialize_null;
  const Str tmp = replace(in, ' ', serialize_separator);
  return tmp.size() ? tmp : serialize_null;
}

Str & operator >> (Str & in, Str & out) throw (Str)
{
  if (! serialize(&in, &out)) throw in;
  return in;
}

Str & operator << (Str & out, const Str & in)
{
  if (out.size()) out += ' ';
  out += serialize(in);
  return out;
}

Str & operator >> (Str & in, bool & out) throw (Str)
{
  if (! serialize(&in, &out, label_bool())) throw in;
  return in;
}

Str & operator << (Str & out, bool in)
{
  if (out.size()) out += ' ';
  out += label_bool()[in];
  return out;
}

Str & operator >> (Str & in, int & out) throw (Str)
{
  Str tmp;
  in >> tmp;
  out = str_to_int(tmp);
  return in;
}

Str & operator << (Str & out, int in)
{
  if (out.size()) out += ' ';
  out += int_to_str(in);
  return out;
}

Str & operator >> (Str & in, uint & out) throw (Str)
{
  Str tmp;
  in >> tmp;
  out = str_to_uint(tmp);
  return in;
}

Str & operator << (Str & out, uint in)
{
  if (out.size()) out += ' ';
  out += uint_to_str(in);
  return out;
}

Str & operator >> (Str & in, double & out) throw (Str)
{
  Str tmp;
  in >> tmp;
  out = str_to_float(tmp);
  return in;
}

Str & operator << (Str & out, double in)
{
  if (out.size()) out += ' ';
  out += float_to_str(in);
  return out;
}

const List<Str> & label_bool()
{
  static const List<Str> data = List<Str>() <<
    "False" <<
    "True";
  return data;
}

}
}

