/*------------------------------------------------------------------------------
 lib/djv_image/io.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
//#define _X_PLUGIN

#include <djv_image/io.h>
#include <djv_image/io_cineon.h>
#include <djv_image/io_dpx.h>
//#include <djv_image/io_iff.h>
#include <djv_image/io_ifl.h>
#include <djv_image/io_lut.h>
#include <djv_image/io_pic.h>
#include <djv_image/io_ppm.h>
#include <djv_image/io_rla.h>
#include <djv_image/io_sgi.h>
#include <djv_image/io_tga.h>
#include <djv_base/sys.h>

namespace djv_image {
namespace io {

//const List<Str> Plugin::_null_list;

Load::Cache::Cache() {}

Load::Cache::Cache(const File & file, int frame, int layer, data::PROXY proxy) :
  file(file),
  frame(frame),
  layer(layer),
  proxy(proxy)
{}

bool Load::Cache::operator == (const Cache & in) const
{
  return
    in.file == file &&
    in.frame == frame &&
    in.layer == layer &&
    in.proxy == proxy;
};

void Load::open(const File & in, Info * info, Tag * tag, Time * time)
  throw (Error)
{
  List<Info> list;
  open(in, &list, tag, time);
  *info = list[0];
}

void Load::open(const File & in, List<Info> * info, Tag * tag, Time * time)
  throw (Error)
{
  Info _info;
  open(in, &_info, tag, time);
  info->push_back(_info);
}

Load_Factory::Load_Factory()
{
  DJV_DBG_OBJ("Load_Factory::Load_Factory");
  
  // Add built-in image plugins.
  
  add(new io_cineon::Load);
  add(new io_dpx::Load);
  //add(new io_iff::Load);
  add(new io_ifl::Load);
  add(new io_lut::Load);
  add(new io_pic::Load);
  add(new io_ppm::Load);
  add(new io_rla::Load);
  add(new io_sgi::Load);
  add(new io_tga::Load);
  
  // Search for image plugins.
  
#if ! defined(_X_PLUGIN)
  set(sys::search_path());
#endif
}

Load_Factory::~Load_Factory()
{
  DJV_DBG_OBJ("Load_Factory::~Load_Factory");
}

namespace {

bool extension_compare(const Str & in, const List<Str> & list)
{
  for (uint i = 0; i < list.size(); ++i)
    if (str::compare(in, list[i]))
      return true;
  return false;
}

}

bool Load_Factory::valid(const File & in, const Str & plugin) const
{
  const List<Load *> & list = this->list();
  for (uint i = 0; i < list.size(); ++i)
  {
    if (plugin.size())
      if (! str::compare(plugin, list[i]->name()))
        continue;
    if (extension_compare(in.extension, list[i]->extension_list()))
      return true;
  }
  return false;
}

Load * Load_Factory::get(const File & in, Info * info, Tag * tag, Time * time,
  const Str & plugin) const throw (Error)
{
  List<Info> _info;
  Load * out = get(in, &_info, tag, time, plugin);
  if (info && _info.size()) *info = _info[0];
  return out;
}

Load * Load_Factory::get(const File & in, List<Info> * info, Tag * tag,
  Time * time, const Str & plugin) const throw (Error)
{
  DJV_DBG_OBJ("Load_Factory::get");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("extension = " << in.extension);
  DJV_DBG_PRN("plugin = " << plugin);

  const List<Load *> & list = this->list();
  for (uint i = 0; i < list.size(); ++i)
  {
    if (plugin.size())
    {
      if (! str::compare(plugin, list[i]->name())) continue;
    }
    else
    {
      if (! extension_compare(in.extension, list[i]->extension_list()))
        continue;
    }
    
    std::auto_ptr<Load> plugin(static_cast<Load *>(list[i]->copy()));
    Tag _tag;
    Time _time;
    plugin->open(in, info, tag ? tag : &_tag, time ? time : &_time);
    return plugin.release();
  }
  
  throw Error(error, error_unrecognized);
  return 0;
}

bool Load_Factory::option(const Str & plugin, const Str & option, Str * data)
{
  DJV_DBG_OBJ("Load_Factory::option");
  DJV_DBG_PRN("plugin = " << plugin);
  DJV_DBG_PRN("option = " << option);
  DJV_DBG_PRN("data = " << *data);
  
  const List<Load *> & list = this->list();
  for (uint i = 0; i < list.size(); ++i)
    if (str::compare(plugin, list[i]->name()))
    {
      const List<Str> option_list = list[i]->option_list();
      for (uint j = 0; j < option_list.size(); ++j)
        if (str::compare(option, option_list[j]))
          return list[i]->option(option, data);
    }
  return false;
}

List<Load *> Load_Factory::list() const
{
  const List<plugin::Plugin *> & list = plugin::Factory::list();
  List<Load *> out(list.size());
  for (uint i = 0; i < list.size(); ++i)
    out[i] = static_cast<Load *>(list[i]);
  return out;
}

Save_Factory::Save_Factory()
{
  DJV_DBG_OBJ("Save_Factory::Save_Factory");
  
  // Add built-in image plugins.
  
  add(new io_cineon::Save);
  add(new io_dpx::Save);
  //add(new io_iff::Save);
  add(new io_lut::Save);
  add(new io_ppm::Save);
  add(new io_sgi::Save);
  add(new io_tga::Save);
  
  // Search for image plugins.
  
#if ! defined(_X_PLUGIN)
  set(sys::search_path());
#endif
}

Save_Factory::~Save_Factory()
{
  DJV_DBG_OBJ("Save_Factory::~Save_Factory");
}

Save * Save_Factory::get(const File & in, const Info & info, const Tag & tag,
  const Time & time, const Str & plugin) const throw (Error)
{
  DJV_DBG_OBJ("Save_Factory::get");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("info = " << info);
  DJV_DBG_PRN("time = " << time);
  DJV_DBG_PRN("plugin = " << plugin);
  
  const List<Save *> & list = this->list();
  for (uint i = 0; i < list.size(); ++i)
  {
    if (plugin.size())
    {
      if (! str::compare(plugin, list[i]->name())) continue;
    }
    else
    {
      if (! extension_compare(in.extension, list[i]->extension_list()))
        continue;
    }
    
    std::auto_ptr<Save> plugin(static_cast<Save *>(list[i]->copy()));
    plugin->open(in, info, tag, time);
    return plugin.release();
  }

  throw Error(error, error_unrecognized);
  return 0;
}

bool Save_Factory::option(const Str & plugin, const Str & option, Str * data)
{
  DJV_DBG_OBJ("Save_Factory::option");
  DJV_DBG_PRN("plugin = " << plugin);
  DJV_DBG_PRN("option = " << option);
  DJV_DBG_PRN("data = " << *data);

  const List<Save *> & list = this->list();
  for (uint i = 0; i < list.size(); ++i)
    if (str::compare(plugin, list[i]->name()))
    {
      const List<Str> option_list = list[i]->option_list();
      for (uint j = 0; j < option_list.size(); ++j)
        if (str::compare(option, option_list[j]))
          return list[i]->option(option, data);
    }
  return false;
}

List<Save *> Save_Factory::list() const
{
  const List<plugin::Plugin *> & list = plugin::Factory::list();
  List<Save *> out(list.size());
  for (uint i = 0; i < list.size(); ++i)
    out[i] = static_cast<Save *>(list[i]);
  return out;
}

Global::Global() :
  _load(std::auto_ptr<Load_Factory>(new Load_Factory)),
  _save(std::auto_ptr<Save_Factory>(new Save_Factory))
{}

Global::~Global() {}

bool Global::valid(const File & in, const Str & plugin) const
  { return _load->valid(in, plugin); }

Load * Global::load(const File & in, Info * info, Tag * tag, Time * time,
  const Str & plugin) const throw (Error)
  { return _load->get(in, info, tag, time, plugin); }

Load * Global::load(const File & in, List<Info> * info, Tag * tag, Time * time,
  const Str & plugin) const throw (Error)
  { return _load->get(in, info, tag, time, plugin); }

Save * Global::save(const File & in, const Info & info, const Tag & tag,
  const Time & time, const Str & plugin) const throw (Error)
  { return _save->get(in, info, tag, time, plugin); }

Load_Factory * Global::load() const { return _load.get(); }
Save_Factory * Global::save() const { return _save.get(); }

}
}
