/*------------------------------------------------------------------------------
 lib/djv_ui/shortcut.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/button.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/label.h>
#include <djv_ui/scroll.h>
#include <djv_ui/shortcut.h>
#include <djv_ui/text.h>
#include <FL/Fl.h>

namespace djv_ui {
namespace shortcut {

struct _Callback_Data
{
  _Callback_Data(widget::Widget * widget, int value) :
    widget(widget),
    value(value)
  {}

  widget::Widget * widget;
  int value;
};

namespace {

class _Edit : public text::Edit
{
public:

  _Edit(int = 0);
  
  void set(int);
  int get() const { return _value; }
  Signal<const _Callback_Data &> sig;
  
private:

  DJV_CALLBACK(_Edit, widget_callback, const Str &);

  void widget_update();
  
  int _value;
};

class _Button : public button::Tool
{
public:

  _Button();
  
  Signal<const _Callback_Data &> sig;
  
  int handle(int);
  
private:

  DJV_CALLBACK(_Button, widget_callback, bool);
};

_Edit::_Edit(int value) :
  sig(this),
  _value(value)
{
  frame(FL_NO_BOX);
  
  widget_update();
  
  Edit::sig.set(this, widget_callback);
}

void _Edit::set(int value)
{
  if (value == _value) return;
  
  DJV_DBG_OBJ("_Edit::set");
  DJV_DBG_PRN("value = " << value);
  
  _value = value;
  
  widget_update();
  sig.emit(_Callback_Data(this, _value));
}

bool alt(int in)
{
  return 
    in == FL_Shift_L ||
    in == FL_Shift_R ||
    in == FL_Alt_L ||
    in == FL_Alt_R ||
    in == FL_Control_L ||
    in == FL_Control_R ||
    in == FL_Meta_L ||
    in == FL_Meta_R;
}

void _Edit::widget_callback(const Str & in)
{
  int value = 0;
  Str tmp = in;
  shortcut::serialize(&tmp, &value);
  set(value);

  widget_update();
}

void _Edit::widget_update()
{
  callbacks(false);
  
  Edit::set(shortcut::serialize(_value));

  callbacks(true);
}

_Button::_Button() :
  Tool("tool_add"),
  sig(this)
{
  type(TOGGLE);
  
  Tool::sig.set(this, widget_callback);
}

void _Button::widget_callback(bool in)
{
  take_focus();
}

int _Button::handle(int in)
{
  switch (in)
  {
    case FL_KEYBOARD:
    {
      int key = Fl::event_key();
      if (get() && ! alt(key))
      {
        if (Fl::event_shift()) key |= FL_SHIFT;
        if (Fl::event_alt()) key |= FL_ALT;
        if (Fl::event_ctrl()) key |= FL_CTRL;
        set(false);
        sig.emit(_Callback_Data(this, key));
        return 1;
      }
    } break;
    
    case FL_FOCUS:
      if (get()) return 1;
      break;

    case FL_UNFOCUS:
      set(false);
      return 1;
  }
  return Tool::handle(in);
}

}

struct Widget::_Widget
{
  scroll::Widget * scroll;
  Layout_V * layout;
  List<label::Widget *> label;
  List<_Edit *> edit;
  List<_Button *> button;
};

Widget::Widget() :
  sig(this),
  _p(new _Widget)
{
  _p->scroll = new scroll::Widget;

  Layout_V * layout = new Layout_V(this);
  layout->margin(0);
  layout->add(_p->scroll);
  
  _p->layout = new Layout_V(_p->scroll);
  _p->layout->spacing(1);
  
  widget_update();
  value_update();
}

Widget::~Widget()
{}

void Widget::set(const List<Shortcut> & in)
{
  if (in == _list) return;
  
  _list = in;
  
  widget_update();
  value_update();
  sig.emit(_list);
}

const List<Shortcut> & Widget::get() const { return _list; }

void Widget::widget_callback(const _Callback_Data & in)
{
  for (uint i = 0; i < _list.size(); ++i)
    if (in.widget == _p->edit[i] || in.widget == _p->button[i])
    {
      _list[i].value = in.value;
      _p->edit[i]->set(_list[i].value);
      break;
    }

  value_update();
  sig.emit(_list);
}

void Widget::widget_update()
{
  DJV_DBG_OBJ("Widget::widget_update");
  
  callbacks(false);
  
  _p->layout->del();
  _p->edit.clear();
  _p->button.clear();
  
  List<djv_ui::layout::Item *> layout_group;
  for (uint i = 0; i < _list.size(); ++i)
  {
    label::Widget * label = new label::Widget(_list[i].name);
    _Edit * edit = new _Edit(_list[i].value);
    _Button * button = new _Button;

    layout_group += label;
    Layout_H * layout = new Layout_H(_p->layout);
    layout->margin(0);
    layout->add(label);
    layout->add(edit);
    layout->add(button);

    edit->sig.set(this, widget_callback);
    button->sig.set(this, widget_callback);

    _p->label += label;
    _p->edit += edit;
    _p->button += button;
  }
  _p->layout->add_stretch();
  djv_ui::layout::group(layout_group);
  
  callbacks(true);
}

void Widget::value_update()
{
  callbacks(false);
  
  for (uint i = 0; i < _list.size(); ++i)
  {
    _p->edit[i]->color(FL_BACKGROUND2_COLOR);
    _p->edit[i]->text_color(text_color());
  }
  
  for (uint i = 0; i < _list.size(); ++i)
    for (uint j = 0; j < _list.size(); ++j)
      if (
        i != j &&
        _list[i].value &&
        _list[i].value == _list[j].value
      ) {
        _p->edit[i]->color(FL_SELECTION_COLOR);
        _p->edit[i]->text_color(FL_BLACK);
        break;
      }
  
  callbacks(true);
}

}
}

