/*------------------------------------------------------------------------------
 lib/djv_ui/window.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/app.h>
#include <djv_ui/window.h>
#include <FL/Fl.H>

namespace djv_ui {
namespace window {

Widget::Widget(const Str & title) :
  Fl_Double_Window(0, 0, 0, 0),
  close_sig(this),
  _title(title),
  _fullscreen(false),
  _resizable(true)
{
  DJV_DBG_OBJ(str::Format("Widget::Widget(%%)").arg(int(this)));

  Fl_Double_Window::end();
  Fl_Double_Window::label(_title.c_str());
  
  callback(close_callback, this);

  app::global()->window_add(this);
}

Widget::~Widget()
{
  DJV_DBG_OBJ(str::Format("Widget::~Widget(%%)").arg(int(this)));
}

void Widget::del()
{
  DJV_DBG_OBJ("Widget::del");

  widget::Base::del();

  app::global()->window_del(this);
}

void Widget::title(const Str & in)
{
  _title = in;
  
  Fl_Double_Window::label(_title.c_str());
}
  
const Str & Widget::title() const { return _title; }

void Widget::fullscreen(bool in)
{
  if (in == _fullscreen) return;

  DJV_DBG_OBJ("Widget::fullscreen");
  DJV_DBG_PRN("in = " << in);

  _fullscreen = in;
  
  if (_fullscreen)
  {
    _geom_tmp = Box2i(Vec2i(x(), y()), geom().size);
    
    Fl_Double_Window::fullscreen();
  }
  else
  {
    fullscreen_off(_geom_tmp.x, _geom_tmp.y, _geom_tmp.w, _geom_tmp.h);
    
    if (_resizable)
    {
      const Vec2i min = size_min();
      size_range(min.x, min.y, 0, 0);
    }
    else
    {
      size_range(_geom_tmp.w, _geom_tmp.h, _geom_tmp.w, _geom_tmp.h);
    }
    
    free_position();
  }
}
  
bool Widget::fullscreen() const { return _fullscreen; }

int Widget::handle(int in)
{
  //DJV_DBG_OBJ("Widget::handle");
  //DJV_DBG_PRN("in = " << in);
  
  switch (widget::Base::handle(in))
  {
    case -1: return 0;
    case 1: return 1;
  }
  //switch (in)
  //{
  //  case FL_PUSH: Fl::focus(this); break;
  //}
  return Fl_Double_Window::handle(in);
}

void Widget::show()
{
  Fl_Double_Window::show();
  widget::Base::show();
}

void Widget::draw()
{
  Fl_Double_Window::draw();
  widget::Base::draw();
}

void Widget::dirty()
{
  DJV_DBG_OBJ(str::Format("Widget::dirty(%%)").arg(int(this)));
  
  widget::Base::dirty();
}

void Widget::update()
{
  DJV_DBG_OBJ(str::Format("Widget::update(%%)").arg(int(this)));
  DJV_DBG_PRN("resizable = " << _resizable);

  dirty();
  
  const Vec2i min = size_min();
  const Vec2i size = clamp(geom().size, min, Vec2i(Fl::w(), Fl::h()));
  DJV_DBG_PRN("min = " << min);
  DJV_DBG_PRN("size = " << size);

  Fl_Double_Window::size(size.x, size.y);
  if (_resizable)
    size_range(min.x, min.y, 0, 0);
  else
    size_range(size.x, size.y, size.x, size.y);
  widget::Base::geom(Box2i(size));
  
  redraw();
}

void Widget::size(const Vec2i & in, bool resizable)
{
  DJV_DBG_OBJ(str::Format("Widget::size(%%)").arg(int(this)));
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("resizable = " << resizable);

  dirty();
  
  const Vec2i min = math::max(size_min(), Vec2i(16, 16));
  const Vec2i size = clamp(in, min, Vec2i(Fl::w(), Fl::h()));
  DJV_DBG_PRN("min = " << min);
  DJV_DBG_PRN("size = " << size);
  
  if (_fullscreen)
  {
    fullscreen_off(_geom_tmp.x, _geom_tmp.y, size.x, size.y);
    _fullscreen = false;
  }
  else
  {
    Fl_Double_Window::size(size.x, size.y);
  }

  _resizable = resizable;
  if (_resizable)
    size_range(min.x, min.y, 0, 0);
  else
    size_range(size.x, size.y, size.x, size.y);

  free_position();
}

void Widget::geom(const Box2i & in)
{
  DJV_DBG_OBJ(str::Format("Widget::geom(%%)").arg(int(this)));
  DJV_DBG_PRN("in = " << in);

  const bool dirty = in.size != geom().size;
  DJV_DBG_PRN("dirty = " << dirty);
  
  Fl_Double_Window::resize(in.x, in.y, in.w, in.h);
  
  if (dirty)
  {
    widget::Base::geom(Box2i(in.size));
    //if (layout()) layout()->dbg();
  }
}

void Widget::resize(int x, int y, int w, int h)
{
  geom(Box2i(x, y, w, h));
}

void Widget::size(layout::SIZE x, layout::SIZE y)
{
  widget::Base::size(x, y);
}

const Box2i & Widget::geom() const { return widget::Base::geom(); }

int Widget::visible() const { return Fl_Double_Window::visible(); }

const layout::SIZE * Widget::size() const
{
  return widget::Base::size();
}

void Widget::close_callback()
{
  hide();
  
  close_sig.emit(true);
}

}
}
